interface IDisposable {
  dispose(): boolean | void;
}
type UnsafeAny = any;
/**
 * Checks if the given value is an instance of `IDisposable`.
 * Returns true only if value is a non null object and has a method "dispose"
 * @param value The value to check.
 * @returns true if the value is an instance of `IDisposable`, false otherwise.
 */
declare const isDisposable: (value: unknown) => value is IDisposable;
/**
 * Disposes the given IDisposable instance.
 * @param disposable The object to dispose.
 * @returns true if the object was disposed, false otherwise.
 */
declare const dispose: (disposable: IDisposable | null | undefined) => boolean;
/**
 * Disposes the IDisposable passed as "this".
 * @example
 * ```ts
 * const disposable: IDisposable = ...;
 * const disposeFn = disposeThis.bind(disposable);
 * disposeFn();
 * ```
 * @returns true if the object was disposed, false otherwise.
 */
declare function disposeThis(this: IDisposable | null | undefined): boolean;
/**
 * Tries to dispose the given object.
 * Does not throw and eats the exception if any.
 * @param disposable The object to dispose or a promise of the object to dispose.
 * @returns true if the object was disposed, false otherwise.
 */
declare const tryDispose: (
  disposable: IDisposable | null | undefined
) => boolean;
declare const using: {
  <
    TFn extends (disposable: TDisposable) => UnsafeAny,
    TDisposable extends Readonly<IDisposable> | null | undefined = IDisposable
  >(
    disposable: TDisposable,
    what: TFn
  ): ReturnType<TFn> extends never
    ? never
    : ReturnType<TFn> extends PromiseLike<Awaited<infer TReturn>>
    ? Promise<TReturn>
    : ReturnType<TFn>;
  <
    TValue,
    TDisposable extends Readonly<IDisposable> | null | undefined = IDisposable
  >(
    disposable: TDisposable,
    what: TValue
  ): TValue extends never
    ? never
    : TValue extends PromiseLike<Awaited<infer TReturn>>
    ? Promise<TReturn>
    : TValue;
};

/**
 * In browser, roaring library initialization is done asynchronously, this method returns true after roaring library WASM is initialized.
 * The library cannot be used until this function returns true.
 * You can await initialization with roaringLibraryInitialize function that returns a promise.
 * @returns true if roaring library WASM is initialized.
 */
declare const roaringLibraryIsReady: () => boolean;
/**
 * In browser, roaring library initialization is done asynchronously, this method returns a promise that resolves when roaring library WASM is initialized.
 * The library cannot be used until this promise is resolved.
 * @returns a promise that resolves when roaring library WASM is initialized.
 * @example
 * await roaringLibraryInitialize();
 * const bitmap = new RoaringBitmap32([123]);
 * console.log(bitmap.toArray()); // [123]
 */
declare const roaringLibraryInitialize: () => Promise<void>;

/**
 * Array of bytes allocted directly in roaring library WASM memory.
 * Note: Memory is not garbage collected, you are responsible to free the allocated memory calling "dispose" method.
 */
declare class RoaringUint8Array
  extends RoaringAllocatedMemory
  implements Iterable<number>
{
  /**
   * The type of typed array used by this class.
   * For RoaringUint8Array is Uint8Array.
   */
  static readonly TypedArray: typeof Uint8Array;
  /**
   * The size in bytes of each element in the array.
   * For RoaringUint8Array is always 1
   */
  static readonly BYTES_PER_ELEMENT: 1;
  /**
   * The type of typed array used by this class.
   * For RoaringUint8Array is Uint8Array.
   */
  readonly TypedArray: typeof Uint8Array;
  /**
   * The size in bytes of each element in the array.
   * For RoaringUint8Array is always 1
   */
  readonly BYTES_PER_ELEMENT: 1;
  /**
   * The ArrayBuffer instance referenced by the array.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned buffer for short periods of time.
   */
  get buffer(): ArrayBuffer;
  /**
   * The length in bytes of the array.
   */
  get length(): number;
  /**
   * The length in bytes of the array.
   */
  get size(): number;
  /**
   * The full WASM heap in hich this array is allocated.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned array for short periods of time.
   */
  get heap(): Uint8Array;
  /**
   * The position in this.heap where the array starts.
   * Is byteOffset.
   * @see byteOffset
   * @see heap
   */
  get heapOffset(): number;
  /**
   * Allocates an array in the roaring WASM heap.
   *
   * Note: Memory is not garbage collected, you are responsible to free the allocated memory calling "dispose" method.
   *
   * If the parameter is a number, it creates a new uninitialized array of the given length.
   * If the parameter is an Iterable, it creates a copy of the given iterable.
   *
   * @param lengthOrArray - Length of the array to allocate or the array to copy
   */
  constructor(
    lengthOrArray?:
      | number
      | Iterable<number>
      | ArrayLike<number>
      | null
      | undefined,
    arenaAllocator?: RoaringArenaAllocator | null | undefined
  );
  /**
   * Throws an error if the memory was freed.
   */
  throwIfDisposed(): void | never;
  /**
   * Writes the given array at the specified position
   * @param array - A typed or untyped array of values to set.
   * @param offset - The index in the current array at which the values are to be written.
   */
  set(array: Iterable<number>, offset?: number): this;
  /**
   * Gets a new JS typed array instance that shares the memory used by this buffer.
   * Note that the buffer may point to an outdated WASM memory if the WASM allocated memory grows while using the returned buffer.
   * Use the returned array for short periods of time.
   *
   * @returns A new typed array that shares the memory with this array.
   */
  asTypedArray(): Uint8Array;
  /**
   * Gets a new NodeJS Buffer instance that shares the memory used by this buffer.
   * Note that the buffer may point to an outdated WASM memory if the WASM allocated memory grows while using the returned buffer.
   * Use the returned array for short periods of time.
   *
   * @returns A new instance of NodeJS Buffer
   */
  asNodeBuffer(): Buffer;
  /**
   * Copies the content of this buffer to a typed array.
   * The returned array is garbage collected and don't need to be disposed manually.
   *
   * @returns A new typed array that contains a copy of this buffer
   */
  toTypedArray(): Uint8Array;
  /**
   * Copies the content of this buffer to a NodeJS Buffer.
   * The returned buffer is garbage collected and don't need to be disposed manually.
   *
   * @returns A new instance of NodeJS Buffer that contains a copy of this buffer
   */
  toNodeBuffer(): Buffer;
  /**
   * Copies the content of this typed array into a standard JS array of numbers and returns it.
   *
   * @returns A new array.
   */
  toArray(): number[];
  /**
   * Returns a string representation of an array.
   */
  toString(): string;
  /**
   * Iterator that iterates through all values in the array.
   */
  [Symbol.iterator](): IterableIterator<number>;
  /**
   * The at() method takes an integer value and returns the item at that index, allowing for positive and negative integers. Negative integers count back from the last item in the array.
   * Follows the specification for array.at().
   * If the computed index is less than 0, or equal to length, undefined is returned.
   * @param index - Zero-based index of the array element to be returned, converted to an integer. Negative index counts back from the end of the array — if index < 0, index + array.length is accessed.
   * @returns The element in the array matching the given index. Always returns undefined if index < -array.length or index >= array.length without attempting to access the corresponding property.
   */
  at(index: number): number | undefined;
  /**
   * Sets the value at the given index.
   * @param index - Zero-based index of the array element to be set, converted to an integer. Negative index counts back from the end of the array — if index < 0, index + array.length is accessed.
   * @param value - The value to set at the given index.
   * @returns True if the value was set, false if the index is out of bounds.
   */
  setAt(index: number, value: number): boolean;
}

/**
 * Array of unsigned 32 bit integers allocted directly in roaring library WASM memory.
 * Note: Memory is not garbage collected, you are responsible to free the allocated memory calling "dispose" method.
 */
declare class RoaringUint32Array
  extends RoaringAllocatedMemory
  implements Iterable<number>
{
  /**
   * The type of typed array used by this class.
   * For RoaringUint32Array is Uint32Array.
   */
  static readonly TypedArray: typeof Uint32Array;
  /**
   * The size in bytes of each element in the array.
   * For RoaringUint32Array is always 4
   */
  static readonly BYTES_PER_ELEMENT: 4;
  /**
   * The type of typed array used by this class.
   * For RoaringUint32Array is Uint32Array.
   */
  readonly TypedArray: typeof Uint32Array;
  /**
   * The size in bytes of each element in the array.
   * For RoaringUint32Array is always 4
   */
  readonly BYTES_PER_ELEMENT: 4;
  /**
   * The ArrayBuffer instance referenced by the array.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned buffer for short periods of time.
   */
  get buffer(): ArrayBuffer;
  /**
   * The length in 32 bit integers of the array.
   * Is byteLength / 4
   * @see byteLength
   */
  get length(): number;
  /**
   * The position in this.heap where the array starts.
   * Is byteOffset / 4
   * @see byteOffset
   * @see heap
   */
  get heapOffset(): number;
  /**
   * The full WASM heap in hich this array is allocated.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned array for short periods of time.
   */
  get heap(): Uint32Array;
  /**
   * Allocates an array in the roaring WASM heap.
   *
   * Note: Memory is not garbage collected, you are responsible to free the allocated memory calling "dispose" method.
   *
   * If the parameter is a number, it creates a new uninitialized array of the given length.
   * If the parameter is an Iterable, it creates a copy of the given iterable.
   *
   * @param lengthOrArray - Length of the array to allocate or the array to copy
   */
  constructor(
    lengthOrArray?:
      | number
      | Iterable<number>
      | ArrayLike<number>
      | null
      | undefined,
    arenaAllocator?: RoaringArenaAllocator | null | undefined
  );
  /**
   * Throws an error if the memory was freed.
   */
  throwIfDisposed(): void | never;
  /**
   * Writes the given array at the specified position
   * @param array - A typed or untyped array of values to set.
   * @param offset - The index in the current array at which the values are to be written.
   */
  set(array: Iterable<number>, offset?: number): this;
  /**
   * Gets a new JS typed array instance that shares the memory used by this buffer.
   * Note that the buffer may point to an outdated WASM memory if the WASM allocated memory grows while using the returned buffer.
   * Use the returned array for short periods of time.
   *
   * @returns A new typed array that shares the memory with this array.
   */
  asTypedArray(): Uint32Array;
  /**
   * Gets a new NodeJS Buffer instance that shares the memory used by this buffer.
   * Note that the buffer may point to an outdated WASM memory if the WASM allocated memory grows while using the returned buffer.
   * Use the returned array for short periods of time.
   *
   * @returns A new instance of NodeJS Buffer
   */
  asNodeBuffer(): Buffer;
  /**
   * Copies the content of this buffer to a typed array.
   * The returned array is garbage collected and don't need to be disposed manually.
   *
   * @returns A new typed array that contains a copy of this buffer
   */
  toTypedArray(): Uint32Array;
  /**
   * Copies the content of this buffer to a NodeJS Buffer.
   * The returned buffer is garbage collected and don't need to be disposed manually.
   *
   * @returns A new instance of NodeJS Buffer that contains a copy of this buffer
   */
  toNodeBuffer(): Buffer;
  /**
   * Copies the content of this typed array into a new JS array of numbers and returns it.
   *
   * @returns A new array.
   */
  toArray(): number[];
  /**
   * Copies the content of this typed array into a new JS Set<number> and returns it.
   *
   * @returns A new array.
   */
  toSet(): Set<number>;
  /**
   * Returns a string representation of an array.
   */
  toString(): string;
  /**
   * Iterator that iterates through all values in the array.
   */
  [Symbol.iterator](): IterableIterator<number>;
  /**
   * The at() method takes an integer value and returns the item at that index, allowing for positive and negative integers. Negative integers count back from the last item in the array.
   * Follows the specification for array.at().
   * If the computed index is less than 0, or equal to length, undefined is returned.
   * @param index - Zero-based index of the array element to be returned, converted to an integer. Negative index counts back from the end of the array — if index < 0, index + array.length is accessed.
   * @returns The element in the array matching the given index. Always returns undefined if index < -array.length or index >= array.length without attempting to access the corresponding property.
   * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/TypedArray/at
   */
  at(index: number): number | undefined;
  /**
   * Sets the value at the given index.
   * @param index - Zero-based index of the array element to be set, converted to an integer. Negative index counts back from the end of the array — if index < 0, index + array.length is accessed.
   * @param value - The value to set at the given index.
   * @returns True if the value was set, false if the index is out of bounds.
   */
  setAt(index: number, value: number): boolean;
  shrink(newLength: number): boolean;
}

/**
 * A Roaring Bitmap that supports 32 bit unsigned integers.
 *
 * The roaring bitmap allocates in WASM memory, remember to dispose
 * the RoaringBitmap32 when not needed anymore to release WASM memory.
 *
 */
declare class RoaringBitmap32 {
  #private;
  /**
   * Creates a new roaring bitmap adding the specified values.
   *
   * The roaring bitmap allocates in WASM memory, remember to dispose
   * the RoaringBitmap32 when not needed anymore to release WASM memory.
   * @param values - The values to add
   */
  constructor(
    valuesOrCapacity?:
      | RoaringBitmap32
      | RoaringUint32Array
      | Iterable<number>
      | ArrayLike<number>
      | number
      | null
      | undefined,
    arenaAllocator?: RoaringArenaAllocator | null
  );
  static from(
    values:
      | RoaringBitmap32
      | RoaringUint32Array
      | Iterable<number>
      | ArrayLike<number>
      | null
      | undefined
  ): RoaringBitmap32;
  /**
   * The RoaringBitmap32.of() static method creates a new Array instance from a variable number of arguments, regardless of number or type of the arguments.
   * Note that is faster to pass a Uint32Array instance instead of an array or an iterable.
   *
   * @static
   * @param values A set of values to add to the new RoaringBitmap32 instance.
   * @returns A new RoaringBitmap32 instance.
   */
  static of(...values: (number | string | null | undefined)[]): RoaringBitmap32;
  /**
   * Creates a new bitmap that contains all the values in the interval: [rangeStart, rangeEnd).
   * Is possible to specify the step parameter to have a non contiguous range.
   *
   * @static
   * @param rangeStart The start index. Trimmed to 0.
   * @param rangeEnd The end index. Trimmed to 4294967296.
   * @param step The increment step, defaults to 1.
   * @returns A new RoaringBitmap32 instance.
   */
  static fromRange(
    rangeStart?: number,
    rangeEnd?: number,
    step?: number
  ): RoaringBitmap32;
  /**
   * Check whether a range of values from rangeStart (included) to rangeEnd (excluded) is present
   *
   * @param {number|undefined} rangeStart The start index (inclusive).
   * @param {number|undefined} [rangeEnd] The end index (exclusive).
   * @returns {boolean} True if the bitmap contains the whole range of values from rangeStart (included) to rangeEnd (excluded), false if not.
   * @memberof ReadonlyRoaringBitmap32
   */
  hasRange(rangeStart?: number, rangeEnd?: number): boolean;
  /**
   * Gets the cardinality (number of elements) between rangeStart (included) to rangeEnd (excluded) of the bitmap.
   * Returns 0 if range is invalid or if no element was found in the given range.
   *
   * @param rangeStart - The start index (inclusive).
   * @param rangeEnd - The end index (exclusive).
   * @returns The number of elements between rangeStart (included) to rangeEnd (excluded).
   */
  rangeCardinality(rangeStart?: number, rangeEnd?: number): number;
  /**
   * Adds all the values in the interval: [rangeStart, rangeEnd).
   *
   * First element is included, last element is excluded.
   * The number of added values is rangeEnd - rangeStart.
   *
   * Areas outside the range are passed through unchanged.
   *
   * @param rangeStart - The start index. Trimmed to 0.
   * @param rangeEnd - The end index. Trimmed to 4294967296.
   * @returns This RoaringBitmap32 instance.
   */
  addRange(rangeStart?: number, rangeEnd?: number): this;
  /**
   * Removes all the values in the interval: [rangeStart, rangeEnd).
   *
   * First element is included, last element is excluded.
   * The number of renived values is rangeEnd - rangeStart.
   *
   * Areas outside the range are passed through unchanged.
   * @param rangeStart - The start index. Trimmed to 0.
   * @param rangeEnd - The end index. Trimmed to 4294967296.
   * @returns This RoaringBitmap32 instance.
   * @memberof RoaringBitmap32
   */
  removeRange(rangeStart?: number, rangeEnd?: number): this;
  /**
   * Negates (in place) the roaring bitmap within a specified interval: [rangeStart, rangeEnd).
   *
   * First element is included, last element is excluded.
   * The number of negated values is rangeEnd - rangeStart.
   *
   * Areas outside the range are passed through unchanged.
   *
   * @param {number|undefined} rangeStart The start index. Trimmed to 0.
   * @param {number|undefined} [rangeEnd] The end index. Trimmed to 4294967296.
   * @returns {this} This RoaringBitmap32 instance.
   * @memberof RoaringBitmap32
   */
  flipRange(rangeStart?: number, rangeEnd?: number): this;
  /**
   * Creates a new roaring bitmap deserializing it from a buffer
   *
   * The roaring bitmap allocates in WASM memory, remember to dispose
   * the RoaringBitmap32 when not needed anymore to release WASM memory.
   *
   * @param buffer - The buffer to deserialize
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   * @returns The reulting bitmap. Remember to dispose the instance when finished using it.
   */
  static deserialize(
    buffer: RoaringUint8Array | Uint8Array | Iterable<number>,
    portable?: boolean
  ): RoaringBitmap32;
  /**
   * Utility function that serializes an array of uint32 to a new NodeJS buffer.
   * The returned buffer is automatically garbage collected.
   *
   * @param values - The values to serialize, a bitmap or an iterable of uint32 values.
   * @param portable - If true, serialization is compatible with the Java and Go versions of the library.
   * If false, serialization is compatible with the C version of the library. Default is false.
   * @returns The NodeJS buffer containing the serialized data.
   */
  static serializeArrayToNewBuffer(
    values: RoaringUint32Array | Iterable<number>,
    portable?: boolean
  ): Buffer;
  /**
   * Utility function that deserializes a RoaringBitmap32 serialized in a buffer to an Array<number> of values.
   * The array can be very big, be careful when you use this function.
   *
   * @param buffer - The buffer to deserialize.
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   * @returns All the values in the bitmap.
   */
  static deserializeToArray(
    buffer: RoaringUint8Array | Uint8Array | Iterable<number>,
    portable?: boolean
  ): number[];
  /**
   * Utility function that deserializes a RoaringBitmap32 serialized in a buffer to a Set<number> of values.
   * The array can be very big, be careful when you use this function.
   *
   * @param buffer - The buffer to deserialize.
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   * @returns All the values in the bitmap.
   */
  static deserializeToSet(
    buffer: RoaringUint8Array | Uint8Array | Iterable<number>,
    portable?: boolean
  ): Set<number>;
  /**
   * Returns a new bitmap that is a copy of this bitmap, same as new RoaringBitmap32(copy)
   * @returns A cloned RoaringBitmap32 instance
   */
  clone(): RoaringBitmap32;
  /**
   * Clears the bitmap, removing all values.
   */
  clear(): void;
  /**
   * Overwrite the content of this bitmap with the content of another bitmap.
   * @param other - The other bitmap to copy.
   * @returns This RoaringBitmap32 instance.
   */
  overwrite(other: RoaringBitmap32): this;
  /**
   * addOffset adds the value 'offset' to each and every value in a bitmap, generating a new bitmap in the process.
   * If offset + element is outside of the range [0,2^32), that the element will be dropped.
   *
   * @param input - The input bitmap.
   * @param offset - The offset to add to each element. Can be positive or negative.
   * @returns A new bitmap with the offset added to each element.
   */
  static addOffset(input: RoaringBitmap32, offset: number): RoaringBitmap32;
  /**
   * Creates a new bitmap with the content of the input bitmap but with given range of values flipped.
   * @param input The input bitmap, it will not be modified
   * @param rangeStart The start index (inclusive).
   * @param rangeEnd The end index (exclusive).
   * @returns A new copied bitmap with the range flipped.
   */
  static flipRange(
    input: RoaringBitmap32,
    rangeStart: number,
    rangeEnd: number
  ): RoaringBitmap32;
  /**
   * Returns true if this instance was disposed.
   */
  get isDisposed(): boolean;
  /**
   * Disposes this object freeing all WASM memory associated to it.
   * Is safe to call this method more than once.
   */
  dispose(): boolean;
  /**
   * Throws an exception if this object was disposed before.
   */
  throwIfDisposed(): void | never;
  /**
   * Get the cardinality of the bitmap (number of elements).
   */
  cardinality(): number;
  /**
   * Get the cardinality of the bitmap (number of elements).
   */
  get size(): number;
  /**
   * Returns true if the bitmap has no elements.
   */
  isEmpty(): boolean;
  /**
   * Adds a 32 bit unsigned integer value.
   * Values are unique, this function does nothing if the value already exists.
   */
  add(value: number): void;
  /**
   * Adds a 32 bit unsigned integer value checking if the bitmap changes.
   * Use add() if you don't need to know if something changed.
   * Values are unique, this function does nothing and returns false if the value already exists.
   *
   * @param value - 32 bit unsigned integer to add in the set.
   * @returns True if the bitmap changed, false if not.
   */
  addChecked(value: number): boolean;
  /**
   * Adds multiple values.
   * Using this is faster than calling add() multiple times.
   * Inserting ordered or partially ordered arrays is faster.
   *
   * @param values - The values to add.
   */
  addMany(
    values:
      | RoaringBitmap32
      | RoaringUint32Array
      | Iterable<number>
      | ArrayLike<number>
      | null
      | undefined
  ): void;
  /**
   * Removes a value from the set.
   * If the value does not exists, this function does nothing.
   *
   * @param value - The value to remove.
   */
  remove(value: number): void;
  /**
   * Removes a value from the set checking if the bitmap changes.
   * Use remove() if you don't need to know if something changed.
   * If the value does not exists, this function does nothing and returns false.
   *
   * @param value - 32 bit unsigned integer to remove from the set.
   * @returns True if the bitmap changed, false if not.
   */
  removeChecked(value: number): boolean;
  /**
   * Gets the minimum value stored in the bitmap.
   * If the bitmap is empty, returns 0xFFFFFFFF
   *
   * @returns The minimum 32 bit unsigned integer or 0xFFFFFFFF if empty.
   */
  minimum(): number;
  /**
   * Gets the maximum value stored in the bitmap.
   * If the bitmap is empty, returns 0.
   *
   * @returns The maximum 32 bit unsigned integer or 0 if empty.
   */
  maximum(): number;
  /**
   * Checks whether the given value is contained in the set.
   *
   * @param value - The value to look for.
   * @returns True if value exists in the set, false if not.
   */
  contains(value: number): boolean;
  /**
   * Returns true if the bitmap is subset of the other.
   *
   * @param other - the other bitmap
   * @returns true if the bitmap is subset of the other.
   */
  isSubset(other: RoaringBitmap32): boolean;
  /**
   * Returns true if this bitmap is strict subset of the other.
   *
   * @param other - The other bitmap
   * @returns True if this bitmap is a strict subset of other
   */
  isStrictSubset(other: RoaringBitmap32): boolean;
  /**
   * Converts the bitmap to an array.
   * The array may be very big, use this function with caution.
   * The returned RoaringUint32Array is allocated in WASM memory and not garbage collected,
   * it need to be freed manually calling dispose().
   *
   * @returns The RoaringUint32Array. Remember to manually dispose to free the memory.
   */
  toRoaringUint32Array(): RoaringUint32Array;
  /**
   * Converts the bitmap to a JS array.
   * The resulting array may be very big, use this function with caution.
   *
   * @returns The array containing all values in the bitmap.
   */
  toArray(): number[];
  /**
   * Converts the bitmap to a JS Set<number>.
   * The resulting set may be very big, use this function with caution.
   *
   * @returns The set containing all values in the bitmap.
   */
  toSet(): Set<number>;
  /**
   * Converts the bitmap to a JS Uint32Array.
   * The resulting array may be very big, use this function with caution.
   *
   * @returns The array containing all values in the bitmap.
   */
  toUint32Array(): Uint32Array;
  /**
   * Checks wether two roaring bitmap contains the same data.
   *
   * @param other - The other bitmap.
   * @returns True if the bitmaps contains the same data, false if not.
   */
  equals(other: RoaringBitmap32): boolean;
  /**
   * Optimizes the bitmap releasing unused memory and compressing containers.
   * Returns true if something changed.
   *
   * @returns True if something changed.
   */
  optimize(): boolean;
  /**
   * If the size of the roaring bitmap is strictly greater than rank, then
   * this function returns the element of given rank.
   * Otherwise, it returns NaN.
   *
   * @param rank - Element rank
   * @returns element or NaN
   */
  select(rank: number): number;
  /**
   * The at() method takes an integer value and returns the item at that index,
   * allowing for positive and negative integers. Negative integers count back from the last item in the set.
   *
   * @param index Zero-based index of the array element to be returned, converted to an integer. Negative index counts back from the end of the array — if index < 0, index + array.length is accessed.
   * @returns The element in the set matching the given index. Always returns undefined if index < -array.length or index >= array.length without attempting to access the corresponding property.
   */
  at(index: number): number | undefined;
  /**
   * Finds the index of the nth set element.
   * Returns -1 if not found.
   *
   * @param value - Element value
   * @returns element index or -1 if not found
   */
  indexOf(value: number): number;
  /**
   * Computes the size of the intersection between two bitmaps.
   * Both bitmaps are unchanged.
   *
   * @param other - The other bitmap.
   * @returns Cardinality of the intersection between two bitmaps.
   */
  andCardinality(other: RoaringBitmap32): number;
  /**
   * Computes the size of the union of two bitmaps.
   * Both bitmaps are unchanged.
   *
   * @param other - The other bitmap.
   * @returns Cardinality of the union of two bitmaps.
   */
  orCardinality(other: RoaringBitmap32): number;
  /**
   * Computes the size of the difference (andnot) of two bitmaps.
   * Both bitmaps are unchanged.
   *
   * @param other - The other bitmap.
   * @returns Cardinality of the difference (andnot) of two bitmaps.
   */
  andNotCardinality(other: RoaringBitmap32): number;
  /**
   * Computes the size of the symmetric difference (xor) between two bitmaps.
   * Both bitmaps are unchanged.
   *
   * @param other - The other bitmap.
   * @returns Cardinality of the symmetric difference (xor) of two bitmaps.
   */
  xorCardinality(other: RoaringBitmap32): number;
  /**
   * Intersects this bitmap with another.
   * Removes the elements from this bitmap that don't exists in the other.
   * Stores the result in this bitmap.
   * The provided bitmap is not modified.
   *
   * @param other - The other bitmap.
   */
  andInPlace(other: RoaringBitmap32): void;
  /**
   * Adds the element of the other bitmap into this bitmap.
   * Stores the result in this bitmap.
   * The provided bitmap is not modified.
   *
   * @param other - The other bitmap.
   */
  orInPlace(other: RoaringBitmap32): void;
  /**
   * Computes the difference between two bitmaps.
   * Stores the result in this bitmap.
   * The provided bitmap is not modified.
   *
   * @param other - The other bitmap.
   */
  xorInPlace(other: RoaringBitmap32): void;
  /**
   * Compute the difference between this and the provided bitmap,
   * writing the result in the current bitmap.
   * The provided bitmap is not modified.
   *
   * @param other - The other bitmap.
   */
  andNotInPlace(other: RoaringBitmap32): void;
  /**
   * Returns the number of integers that are smaller or equal to the given value.
   *
   * @param value - The value to rank
   * @returns The number of values smaller than the given value
   */
  rank(value: number): number;
  /**
   * Check whether the two bitmaps intersect (have at least one element in common).
   *
   * @param other - The other bitmap.
   * @returns True if the two bitmaps intersects, false if not.
   */
  intersects(other: RoaringBitmap32): boolean;
  /**
   * Check whether a bitmap and a closed range intersect.
   *
   * @param rangeStart The start of the range.
   * @param rangeEnd The end of the range.
   * @returns boolean True if the bitmap and the range intersects, false if not.
   */
  intersectsWithRange(rangeStart?: number, rangeEnd?: number): boolean;
  /**
   * Computes the Jaccard index between two bitmaps.
   * (Also known as the Tanimoto distance, or the Jaccard similarity coefficient)
   * See https://en.wikipedia.org/wiki/Jaccard_index
   *
   * The Jaccard index is undefined if both bitmaps are empty.
   *
   * @returns The Jaccard index
   */
  jaccardIndex(other: RoaringBitmap32): number;
  /**
   * How many bytes are required to serialize this bitmap.
   *
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   */
  getSerializationSizeInBytes(portable?: boolean): number;
  /**
   * Serializes a bitmap to a byte buffer allocated in WASM memory.
   *
   * The returned RoaringUint8Array is allocated in WASM memory and not garbage collected,
   * it need to be freed manually calling dispose().
   *
   * @param portable - If true, serialization is compatible with the Java and Go versions of the library.
   * If false, serialization is compatible with the C version of the library. Default is false.
   * @returns The RoaringUint8Array. Remember to manually dispose to free the memory.
   */
  serializeToRoaringUint8Array(portable?: boolean): RoaringUint8Array;
  /**
   * Serializes a bitmap to a typed Uint8Array.
   * The returned array is automatically garbage collected and there is no need to be disposed manually.
   *
   * @param portable - If true, serialization is compatible with the Java and Go versions of the library.
   * If false, serialization is compatible with the C version of the library. Default is false.
   * @returns The Uint8Array that contains the serialized bitmap
   */
  serializeToUint8Array(portable?: boolean): Uint8Array;
  /**
   * Serializes a bitmap to a NodeJS buffer.
   * The returned buffer is automatically garbage collected and there is no need to be disposed manually.
   *
   * @param portable - If true, serialization is compatible with the Java and Go versions of the library.
   * If false, serialization is compatible with the C version of the library. Default is false.
   * @returns The NodeJS Buffer that contains the serialized bitmap
   */
  serializeToNodeBuffer(portable?: boolean): Buffer;
  /**
   * Reads a bitmap from a serialized version.
   * Throws an error if deserialization failed.
   *
   * @param buffer - The buffer to deserialize
   * @param portable - If true, deserialization is compatible with the Java and Go versions of the library.
   * If false, deserialization is compatible with the C version of the library. Default is false.
   */
  deserialize(
    buffer: RoaringUint8Array | Uint8Array | Iterable<number>,
    portable?: boolean
  ): void;
  /**
   * Remove run-length encoding even when it is more space efficient.
   *
   * Return whether a change was applied.
   *
   * @returns True if a change was applied, false if not.
   */
  removeRunCompression(): boolean;
  /**
   * Convert array and bitmap containers to run containers when it is more efficient;
   * also convert from run containers when more space efficient.
   *
   * Returns true if the bitmap has at least one run container.
   *
   * Additional savings might be possible by calling shrinkToFit().
   *
   * @returns True if the bitmap has at least one run container.
   */
  runOptimize(): boolean;
  /**
   * If needed, reallocate memory to shrink the memory usage.
   *
   * Returns the number of bytes saved.
   *
   * @returns The number of bytes saved.
   */
  shrinkToFit(): number;
  /**
   * Returns a new RoaringBitmap32 with the intersection (and) between the given two bitmaps.
   *
   * The provided bitmaps are not modified.
   *
   * @param a - The first RoaringBitmap32 instance to and.
   * @param b - The second RoaringBitmap32 instance to and.
   * @returns A new RoaringBitmap32 that contains the intersection a AND b
   */
  static and(a: RoaringBitmap32, b: RoaringBitmap32): RoaringBitmap32;
  /**
   * Returns a new RoaringBitmap32 with the union (or) of the two given bitmaps.
   *
   * The provided bitmaps are not modified.
   *
   * @param a The first RoaringBitmap32 instance to or.
   * @param b The second RoaringBitmap32 instance to or.
   */
  static or(a: RoaringBitmap32, b: RoaringBitmap32): RoaringBitmap32;
  /**
   * Returns a new RoaringBitmap32 with the symmetric union (xor) between the two given bitmaps.
   *
   * The provided bitmaps are not modified.
   *
   * @param a The first RoaringBitmap32 instance to xor.
   * @param b The second RoaringBitmap32 instance to xor.
   */
  static xor(a: RoaringBitmap32, b: RoaringBitmap32): RoaringBitmap32;
  /**
   * Returns a new RoaringBitmap32 with the difference (and not) between the two given bitmaps.
   *
   * The provided bitmaps are not modified.
   *
   * @static
   * @param a The first RoaringBitmap32 instance.
   * @param b The second RoaringBitmap32 instance.
   */
  static andNot(a: RoaringBitmap32, b: RoaringBitmap32): RoaringBitmap32;
  /**
   * Performs a union between all the given array of RoaringBitmap32 instances.
   *
   * This function is faster than calling or multiple times.
   *
   * @param bitmaps - An array of RoaringBitmap32 instances to or together.
   * @returns A new RoaringBitmap32 that contains the union of all the given bitmaps.
   */
  static orMany(
    bitmaps: readonly (RoaringBitmap32 | null | undefined | false)[]
  ): RoaringBitmap32;
  /**
   * Performs a xor between all the given array of RoaringBitmap32 instances.
   *
   * This function is faster than calling xor multiple times.
   *
   * @param bitmaps - An array of RoaringBitmap32 instances to or together.
   * @returns A new RoaringBitmap32 that contains the xor of all the given bitmaps.
   */
  static xorMany(
    bitmaps: readonly (RoaringBitmap32 | null | undefined | false)[]
  ): RoaringBitmap32;
  /**
   * Swaps the content of two RoaringBitmap32 instances.
   *
   * @static
   * @param {RoaringBitmap32} a First RoaringBitmap32 instance to swap
   * @param {RoaringBitmap32} b Second RoaringBitmap32 instance to swap
   * @memberof RoaringBitmap32
   */
  static swap(a: RoaringBitmap32, b: RoaringBitmap32): void;
}

declare class RoaringArenaAllocator {
  #private;
  static get current(): RoaringArenaAllocator | null;
  /**
   * Starts a new arena allocator.
   * @returns The new arena allocator.
   */
  static start(): RoaringArenaAllocator;
  /**
   * Stops the current arena allocator.
   * @returns The stopped arena allocator.
   * @see start
   * @see current
   * @see with
   */
  static stop(): RoaringArenaAllocator | null;
  /**
   * Gets the number of references currently registered.
   * @returns The number of references, escaped or not.
   */
  get size(): number;
  /**
   * Gets the number of references currently escaped.
   * Escaped references are not disposed when the arena is stopped.
   * @returns The number of escaped references.
   * @see escape
   */
  get escaped(): number;
  static with(
    fn: (allocator: RoaringArenaAllocator) => void,
    allocator?: RoaringArenaAllocator
  ): void;
  constructor();
  disposeAll(): number;
  start(): this;
  stop(): this;
  with<T>(fn: (allocator: RoaringArenaAllocator) => T): T;
  register<T extends IDisposable>(disposable: T): T;
  escape<T extends IDisposable>(disposable: T): T;
  newRoaringUint8Array(
    lengthOrArray?:
      | number
      | Iterable<number>
      | ArrayLike<number>
      | null
      | undefined
  ): RoaringUint8Array;
  newRoaringUint32Array(
    lengthOrArray?:
      | number
      | Iterable<number>
      | ArrayLike<number>
      | null
      | undefined
  ): RoaringUint32Array;
  newRoaringBitmap32(
    valuesOrCapacity?:
      | RoaringBitmap32
      | RoaringUint32Array
      | Iterable<number>
      | ArrayLike<number>
      | number
      | null
      | undefined
  ): RoaringBitmap32;
}

declare abstract class RoaringAllocatedMemory {
  #private;
  /**
   * The offset in bytes of the array (the location of the first byte in WASM memory).
   */
  get byteOffset(): number;
  /**
   * The length in bytes of the array.
   * For RoaringUint8Array it is equal to this.length
   * For RoaringUint32Array it is equal to this.length * 4
   */
  get byteLength(): number;
  constructor(
    ptr: number,
    bytes: number,
    arenaAllocator?: RoaringArenaAllocator | null
  );
  /**
   * The ArrayBuffer instance referenced by the array.
   * Note that the buffer may become invalid if the WASM allocated memory grows.
   * When the WASM grows the preallocated memory this property will return the new allocated buffer.
   * Use the returned buffer for short periods of time.
   */
  get buffer(): ArrayBuffer;
  /**
   * Returns true if this object was deallocated.
   */
  get isDisposed(): boolean;
  /**
   * Frees the allocated memory.
   * Is safe to call this method more than once.
   * @returns True if memory gets freed during this call, false if not.
   */
  dispose(): boolean;
  protected toJSON(): unknown;
  /**
   * Decreases the size of the allocated memory.
   * It does nothing if the new length is greater or equal to the current length.
   * If the new length is less than 1, it disposes the allocated memory.
   * NOTE: if the value is non zero, this does not reallocate the consumed memory, it just chances the reported size in byteLength and length properties.
   * @param newLength - The new length in bytes.
   * @returns True if the memory was shrunk, false if not.
   */
  shrink(newLength: number): boolean;
}

export {
  IDisposable,
  RoaringAllocatedMemory,
  RoaringArenaAllocator,
  RoaringBitmap32,
  RoaringUint32Array,
  RoaringUint8Array,
  dispose,
  disposeThis,
  isDisposable,
  roaringLibraryInitialize,
  roaringLibraryIsReady,
  tryDispose,
  using,
};
